/**
 * \par Copyright (C), 2012-2019, Parconier
 * \class  DCMotor
 * \brief   Driver for DC motor device.
 * @file    DCMotor.cpp
 * @author  parconier
 * @version V1.0.0
 * @date    2019/01/31
 * @brief   Driver for DC motor device.
 *
 * \par Copyright
 * This software is Copyright (C), 2012-2016, Parconier. Use is subject to license \n
 * conditions. The main licensing options available are GPL V2 or Commercial: \n
 *
 * \par Open Source Licensing GPL V2
 * This is the appropriate option if you want to share the source code of your \n
 * application with everyone you distribute it to, and you also want to give them \n
 * the right to share who uses it. If you wish to use this software under Open \n
 * Source Licensing, you must contribute all your source code to the open source \n
 * community in accordance with the GPL Version 2 when your application is \n
 * distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * \par Description
 * This file is a drive for DC motor device.
 *
 * \par Method List:
 *
 *    1. void DCMotor::run(int16_t speed)
 *
 * \par History:
 * <pre>
 * `<Author>`         `<Time>`        `<Version>`        `<Descr>`
 * Gonzalo Gutierrez         2019/01/31     1.0.0            Rebuild the old lib.
 * </pre>
 *
 * @example xxxx.ino
 */
#include "DCMotor.h"




/**
 * Constructor
 * \param[in]
 *   None
 */
DCMotor::DCMotor(uint8_t _dir_pin,uint8_t _pwm_pin)
{
  //TODO: Ver de hacer esto de abajo una funcion global para que
  //no se ejecute en todas las instancias
  
  //The PWM frequency is 976 Hz
  //#if defined(__AVR_ATmega328__) // ATmega328
  //TCCR1A = _BV(WGM10);
  //TCCR1B = _BV(CS11) | _BV(CS10) | _BV(WGM12);
  //TCCR2A = _BV(WGM21) | _BV(WGM20);
  //TCCR2B = _BV(CS22);
  //#endif
  

  dir_pin = _dir_pin;
  pwm_pin = _pwm_pin;  
  pinMode(dir_pin, OUTPUT);
}


/**
* Constructor 
* \param[in]
*   motor_id - motor identification (MA or MB)
*/
DCMotor::DCMotor(uint8_t motor_id)
{
  dir_pin = Motor_Slot[motor_id].DIR;
  pwm_pin = Motor_Slot[motor_id].PWM;  
  pinMode(dir_pin, OUTPUT);
}

/**
 * \par Function
 *   run
 * \par Description
 *   Control the motor forward or reverse
 * \param[in]
 *   speed - Speed value from -255 to 255
 * \par Output
 *   None
 * \return
 *   None
 * \par Others
 *   None
 */
void DCMotor::run(int16_t speed)
{  
  if (speed < -255) speed = -255;
  if (speed > 255) speed =  255;
   
  if(speed < 0)
  { 
    digitalWrite(dir_pin, HIGH);
    analogWrite(pwm_pin, -speed);
  }
  else
  {
    digitalWrite(dir_pin, LOW);
    analogWrite(pwm_pin, speed);
  }
}

/**
 * \par Function
 *   stop
 * \par Description
 *   Stop the motor
 * \par Output
 *   None
 * \return
 *   None
 * \par Others
 *   None
 */
void DCMotor::stop(void)
{
  DCMotor::run(0);
}
  
void DCMotor::moveMotor(uint8_t power, uint8_t direction)
{
  //127*log10(power+1)+1
  int val = map(power, 0, 100, 0, 255); 
  if (direction == CLOCKWISE)
  {
    DCMotor::run(val);
  } 
  else 
  {
    DCMotor::run(-val);
  }

}




