/**
 * \par Copyright (C), 2012-2019, Parconier
 * \class  DCMotor
 * \brief   Driver for DC motor device.
 * @file    DCMotor.cpp
 * @author  parconier
 * @version V1.0.0
 * @date    2019/01/31
 * @brief   Driver for DC motor device.
 *
 *
 * \par Open Source Licensing GPL V2
 * This is the appropriate option if you want to share the source code of your \n
 * application with everyone you distribute it to, and you also want to give them \n
 * the right to share who uses it. If you wish to use this software under Open \n
 * Source Licensing, you must contribute all your source code to the open source \n
 * community in accordance with the GPL Version 2 when your application is \n
 * distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * \par Description
 * This file is a drive for DC motor device.
 *
 * \par Method List:
 *
 *    1. void DCMotor::run(int16_t speed)
 *
 * \par History:
 * <pre>
 * `<Author>`         `<Time>`        `<Version>`        `<Descr>`
 * Gonzalo Gutierrez         2019/01/31     1.0.0            Rebuild the old lib.
 * </pre>
 *
 * @example xxxx.ino
 */

#ifndef DCMotor_H
#define DCMotor_H

#include <Arduino.h>

#define MA 					0
#define MB 					1
#define FORWARD 			1
#define BACKWARD 			0
#define COUNTERCLOCKWISE 	1
#define CLOCKWISE 			0


typedef struct
{
  uint8_t motor_id;    
  uint8_t OUTA;   
  uint8_t OUTB;   
  uint8_t PWM;   
  uint8_t DIR;
} Motor_slot_type;

extern Motor_slot_type Motor_Slot[2];  

/**
 * Class: DCMotor
 * \par Description
 * Declaration of Class DCMotor
 */
class DCMotor
{
	public:
		/**
 		* Constructor 
 		* \param[in]
 		*   dir_pin - arduino port for direction pin
 		* \param[in]
 		*   pwm_pin - arduino port for pwm input
 		*/
  		DCMotor(uint8_t dir_pin,uint8_t pwm_pin);

		/**
 		* Constructor 
 		* \param[in]
 		*   motor_id - motor identification (MA or MB)
 		*/
  		DCMotor(uint8_t motor_id);  
		
		/**
		 * \par Function
		 *   run
		 * \par Description
		 *   Control the motor forward or reverse
		 * \param[in]
		 *   speed - Speed value from -255 to 255
		 * \par Output
		 *   None
		 * \return
		 *   None
		 * \par Others
		 *   None
		 */
		void run(int16_t speed);


        /**
         * \par Function
         *   stop
         * \par Description
         *   Stop the motor
         * \par Output
         *   None
         * \return
         *   None
         * \par Others
         *   None
         */
        void stop(void);

		void moveMotor(uint8_t power, uint8_t direction);

		private:
  			volatile uint8_t dir_pin;
  			volatile uint8_t pwm_pin;
  			int16_t  		 last_speed;
};


#endif